 /*
    This program starts up two (or more) threads and the
    threads are executing in the same function. In other
    words, the threads represent separate "paths of execution"
    through the same piece of code. The piece of code in question
    just prints out a single character to standard output. The
    character printed out is a parameter passed to the thread
    just before the thread is started. One thread is passed the
    character 'a' and the other thread is passed 'b'. In the
    output we see long strings of a's and b's that represent
    each thread running uninterupted during it time quantum
    (just as in the last example).

    This program shows that a thread is really just a "path of
    execution" through a piece of code (i.e., the thread is not
    the code, the thread is a path through the code). And each
    thread has its own copy of any variables used in the code.
    In this example, each path (i.e., thread) through the code
    has its own copy of the thread parameter that holds the
    charater that the thread is printing.
*/
#include <windows.h>
#include <stdio.h>

int main()
{
   #define N 2   // number of threads (try changing this value)

   int i;

   DWORD WINAPI thread(LPVOID); //thread function prototype

   int threadParam[N];          // thread function parameters

   // create multiple threads from a single thread function
   printf("Starting %d threads\n", N);
   for (i = 0; i < N; i++)
   {
      threadParam[i] = 'a'+i;
      CreateThread(NULL, 0, thread, &threadParam[i], 0, NULL);
   }

   while(1) // loop forever
   {
      Sleep(1000);  // try commenting out this line
   }

   return 0;
}//main


// This thread function can be used to create an
// arbitrary number of "identical" threads.
DWORD WINAPI thread(LPVOID threadParam)
{
   char myChar = *((char*)threadParam); // keep a local copy of the parameter

   while(1)  // loop forever
   {
      printf("%c", myChar);
   }
   return 0;
}//thread
